from clovers import Event as CloversEvent


class Event:
    def __init__(self, event: CloversEvent):
        self.event: CloversEvent = event

    @property
    def nickname(self) -> str:
        return self.event.properties["nickname"]

    @property
    def group_id(self) -> str:
        return self.event.properties["group_id"]

    @property
    def to_me(self) -> bool:
        return self.event.properties["to_me"]

    @property
    def permission(self) -> int:
        return self.event.properties["permission"]

    @property
    def image_url(self) -> str | None:
        image_list = self.event.properties["image_list"]
        if image_list:
            return image_list[0]



from datetime import datetime, timezone
from pydantic import BaseModel
import hashlib
import hmac
import json
import httpx
from .main import ChatInterface, ChatInfo


class Config(ChatInfo, BaseModel):
    secret_id: str
    secret_key: str


def headers(
    secret_id: str,
    secret_key: str,
    host: str,
    payload: str,
) -> dict:
    algorithm = "TC3-HMAC-SHA256"
    service = "hunyuan"
    version = "2023-09-01"
    action = "ChatCompletions"
    ct = "application/json"
    signed_headers = "content-type;host;x-tc-action"
    now_utc = datetime.now(timezone.utc)
    timestamp = str(int(now_utc.timestamp()))
    date = now_utc.strftime("%Y-%m-%d")
    # 拼接规范请求串
    canonical_request = f"POST\n/\n\ncontent-type:{ct}\nhost:{host}\nx-tc-action:{action.lower()}\n\n{signed_headers}\n{hashlib.sha256(payload.encode('utf-8')).hexdigest()}"
    # 拼接待签名字符串
    credential_scope = f"{date}/{service}/tc3_request"
    string_to_sign = f"{algorithm}\n{timestamp}\n{credential_scope}\n{hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()}"

    # 计算签名
    def sign(key, msg):
        return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

    secret_date = sign(("TC3" + secret_key).encode("utf-8"), date)
    secret_service = sign(secret_date, service)
    secret_signing = sign(secret_service, "tc3_request")
    signature = hmac.new(secret_signing, string_to_sign.encode("utf-8"), hashlib.sha256).hexdigest()
    # 拼接 Authorization
    return {
        "Authorization": f"{algorithm} Credential={secret_id}/{credential_scope}, SignedHeaders={signed_headers}, Signature={signature}",
        "Content-Type": "application/json",
        "Host": host,
        "X-TC-Action": action,
        "X-TC-Timestamp": timestamp,
        "X-TC-Version": version,
    }


class Chat(ChatInterface):
    """腾讯混元"""

    def __init__(self, config: dict) -> None:
        super().init()
        _config = Config.model_validate(config)
        self.model = _config.model
        self.prompt_system = _config.prompt_system
        self.memory = _config.memory
        self.timeout = _config.timeout
        self.url = _config.url
        self.host = self.url.split("//", 1)[1]
        self.secret_id = _config.secret_id
        self.secret_key = _config.secret_key
        self.async_client = httpx.AsyncClient(proxy=_config.proxy)

    @staticmethod
    async def build_content(text: str, image_url: str | None):
        return text

    async def ChatCompletions(self):
        messages = [{"Role": "system", "Content": self.prompt_system}]
        messages.extend({"Role": message["role"], "Content": message["content"]} for message in self.messages)
        payload = {"Model": self.model, "Messages": messages}
        payload = json.dumps({"Model": self.model, "Messages": messages}, separators=(",", ":"), ensure_ascii=False)
        resp = (
            await self.async_client.post(
                self.url,
                headers=headers(
                    secret_id=self.secret_id,
                    secret_key=self.secret_key,
                    host=self.host,
                    payload=payload,
                ),
                content=payload,
            )
        ).raise_for_status()
        return resp.json()["Response"]["Choices"][0]["Message"]["Content"]



from pydantic import BaseModel
import httpx
import base64
from .main import ChatInterface, ChatInfo


class Config(ChatInfo, BaseModel):
    api_key: str


class Chat(ChatInterface):
    """Gemini"""

    def __init__(self, config: dict) -> None:
        super().init()
        _config = Config.model_validate(config)
        self.model = _config.model
        self.prompt_system = _config.prompt_system
        self.memory = _config.memory
        self.timeout = _config.timeout
        self.url = f"{_config.url.rstrip("/")}/{_config.model}:generateContent?key={_config.api_key}"
        self.async_client = httpx.AsyncClient(proxy=_config.proxy)

    async def build_content(self, text: str, image_url: str | None):
        data: list[dict] = [{"text": text}]
        if image_url:
            response = (await self.async_client.get(image_url)).raise_for_status()
            image_data = base64.b64encode(response.content).decode("utf-8")
            data.append({"inline_data": {"mime_type": "image/jpeg", "data": image_data}})
        return data

    async def ChatCompletions(self):
        data = {
            "system_instruction": {"parts": {"text": self.prompt_system}},
            "contents": [
                (
                    {
                        "role": "user",
                        "parts": message["content"],
                    }
                    if message["role"] == "user"
                    else {
                        "role": "model",
                        "parts": [{"text": message["content"]}],
                    }
                )
                for message in self.messages
            ],
        }
        resp = (await self.async_client.post(self.url, json=data, headers={"Content-Type": "application/json"})).raise_for_status()
        return resp.json()["candidates"][0]["content"]["parts"][0]["text"].rstrip("\n")



from datetime import datetime
from abc import ABC, abstractmethod
from typing import Any
from clovers.logger import logger


class AIChat(ABC):
    """对话模型"""

    model: str
    """模型版本名"""

    def __init__(self) -> None:
        self.running: bool = False

    @abstractmethod
    async def chat(self, nickname: str, text: str, image_url: str | None) -> str | None: ...

    @abstractmethod
    def memory_clear(self) -> None: ...

    @abstractmethod
    def set_prompt_system(self, prompt_system: str) -> None: ...


class ChatInfo:
    """对话设置"""

    url: str
    """接入点url"""
    model: str
    """模型版本名"""
    prompt_system: str
    """系统提示词"""
    memory: int
    """对话记录长度"""
    timeout: int | float
    """对话超时时间"""
    proxy: str | None = None
    """代理地址"""


class ChatInterface(ChatInfo, AIChat):
    """模型对话接口"""

    messages: list[dict]
    """对话记录"""
    memory: int
    """对话记录长度"""
    timeout: int | float
    """对话超时时间"""

    def init(self) -> None:
        super().__init__()
        self.messages: list[dict] = []

    @abstractmethod
    def __init__(self, config: dict) -> None: ...

    @abstractmethod
    async def build_content(self, text: str, image_url: str | None) -> Any: ...

    @abstractmethod
    async def ChatCompletions(self) -> str | None: ...

    def memory_filter(self, timestamp: int | float):
        """过滤记忆"""
        self.messages = self.messages[-self.memory :]
        self.messages = [message for message in self.messages if message["time"] > timestamp - self.timeout]
        if self.messages[0]["role"] == "assistant":
            self.messages = self.messages[1:]
        assert self.messages[0]["role"] == "user"

    async def chat(self, nickname: str, text: str, image_url: str | None) -> str | None:
        now = datetime.now()
        timestamp = now.timestamp()
        try:
            contect = await self.build_content(f'{nickname} ({now.strftime("%Y-%m-%d %H:%M")}):{text}', image_url)
        except Exception as err:
            logger.exception(err)
            return
        self.messages.append({"time": timestamp, "role": "user", "content": contect})
        self.memory_filter(timestamp)
        try:
            resp_content = await self.ChatCompletions()
            self.messages.append({"time": timestamp, "role": "assistant", "content": resp_content})
        except Exception as err:
            del self.messages[-1]
            logger.exception(err)
            return
        return resp_content

    def memory_clear(self) -> None:
        self.messages.clear()

    def set_prompt_system(self, prompt_system: str) -> None:
        self.prompt_system = prompt_system



import re
from clovers.plugin import Plugin, Result
from clovers.logger import logger
from collections.abc import Callable
from .clovers import Event
from .config import config_data
from .manager import Manager


plugin = Plugin(
    build_event=lambda event: Event(event),
    build_result=lambda result: Result("text", result),
    priority=100,
)
pattern = re.compile(r"[^\u4e00-\u9fa5a-zA-Z\s]")

type RuleType = Callable[[Event], bool]

permission_check: RuleType = lambda e: e.permission > 0


def new(config: dict) -> None:
    chat_manager = Manager(config)
    if whitelist := chat_manager.whitelist:
        logger.info(f"{chat_manager.name} 检查规则设置为白名单模式：{whitelist}")
        rule: RuleType = lambda event: event.to_me and event.group_id in whitelist
    elif blacklist := chat_manager.blacklist:
        logger.info(f"{chat_manager.name} 检查规则设置为黑名单模式：{blacklist}")
        rule: RuleType = lambda event: event.to_me and event.group_id not in blacklist
    else:
        logger.info(f"{chat_manager.name} 未设置黑白名单，已在全部群组启用")
        rule: RuleType = lambda event: event.to_me

    is_command = False

    @plugin.handle(["记忆清除"], ["group_id", "to_me", "permission"], rule=[rule, permission_check], block=False)
    async def _(event: Event):
        nonlocal is_command
        is_command = True
        chat = chat_manager.chat(event.group_id)
        chat.memory_clear()
        return f"本群【{chat_manager.name} - {chat.model}】记忆已清除！"

    @plugin.handle(["修改设定"], ["group_id", "to_me", "permission"], rule=[rule, permission_check], block=False)
    async def _(event: Event):
        nonlocal is_command
        is_command = True
        prompt_system = event.event.raw_command[4:]
        chat = chat_manager.chat(event.group_id)
        if prompt_system:
            chat.memory_clear()
            chat.set_prompt_system(
                prompt_system
                + '\n你收到的消息格式为 "昵称 (日期 时间):消息" 例如 "小明 (2024-5-31 12:00):你好" 你的回复不应该有昵称，时间和日期。'
                "\n你收到的消息含有用户的昵称 你应该注意在与哪个用户对话 不要让昵称的含义影响到你的回复"
            )
            return f"本群【{chat_manager.name} - {chat.model}】设定已修改！"
        else:
            del chat_manager.chats[event.group_id]
            return f"本群【{chat_manager.name} - {chat.model}】设定已重置！"

    def chat_rule(event: Event):
        nonlocal is_command
        if is_command:
            is_command = False
            return False
        return rule(event)

    @plugin.handle(None, ["group_id", "nickname", "to_me", "image_list"], rule=chat_rule, priority=1, block=False)
    async def _(event: Event):
        chat = chat_manager.chat(event.group_id)
        if chat.running:
            return
        text = event.event.raw_command
        nickname = pattern.sub("", event.nickname) or event.nickname[0]
        chat.running = True
        result = await chat.chat(nickname, text, event.image_url)
        chat.running = False
        return result


for cfg in config_data.config_list:

    _config = {
        "prompt_system": config_data.prompt_system,
        "memory": config_data.memory,
        "timeout": config_data.timeout,
    }
    _config.update(cfg)
    try:
        new(_config)
    except Exception as e:
        logger.exception(e)
        logger.debug(_config)


__plugin__ = plugin



from pydantic import BaseModel
from .ai.main import AIChat
from .ai.mix import Chat as MixChat
from .ai.openai import Chat as OpenAIChat
from .ai.hunyuan import Chat as HunYuanChat
from .ai.gemini import Chat as GeminiChat
from .ai.deepseek import Chat as DeepSeekChat


def matchChat(key: str):
    match key:
        case "chatgpt":
            return OpenAIChat, "ChatGPT"
        case "qwen":
            return OpenAIChat, "通义千问"
        case "deepseek":
            return DeepSeekChat, "DeepSeek"
        case "hunyuan":
            return HunYuanChat, "腾讯混元"
        case "gemini":
            return GeminiChat, "Gemini"
        case _:
            raise ValueError(f"不支持的模型:{key}")


class ManagerInfo:
    """实例设置"""

    whitelist: set[str] = set()
    """白名单"""
    blacklist: set[str] = set()
    """黑名单"""


class ManagerConfig(ManagerInfo, BaseModel):
    pass


class MixManagerConfig(ManagerInfo, BaseModel):
    text: dict
    image: dict


class Manager(ManagerInfo):
    """实例运行管理类"""

    name: str
    """实例名称"""

    def __init__(self, config: dict) -> None:
        self.chats: dict[str, AIChat] = {}
        self.config = config
        if config["key"] == "mix":
            self.name = "图文混合模型"
            _config = MixManagerConfig.model_validate(config)
            ChatText, textchatname = matchChat(_config.text["key"])
            chat_text = ChatText(config | _config.text)
            ChatImage, imagechatname = matchChat(_config.image["key"])
            chat_image = ChatImage(config | _config.image)
            model = f"text:{textchatname}:{chat_text.model} - image:{imagechatname}:{chat_image.model}"

            def newChat(config: dict):
                return MixChat(
                    _config.whitelist,
                    _config.blacklist,
                    chat_text,
                    chat_image,
                    model,
                )

            self.newChat = newChat
        else:
            _config = ManagerConfig.model_validate(config)
            self.newChat, self.name = matchChat(config["key"])
        self.whitelist = _config.whitelist
        self.blacklist = _config.blacklist

    def chat(self, group_id: str):
        if group_id not in self.chats:
            self.chats[group_id] = self.newChat(self.config)
        return self.chats[group_id]



from datetime import datetime
from clovers.logger import logger
from .openai import Chat as OpenAIChat, ChatCompletionMessageParam


class Chat(OpenAIChat):
    """DeepSeek"""

    async def chat(self, nickname: str, text: str, image_url: str | None) -> str | None:
        if text.endswith("--think"):
            think = True
            text = text[:-7].strip()
        else:
            think = False
        now = datetime.now()
        timestamp = now.timestamp()
        try:
            contect = await self.build_content(f'{nickname} ({now.strftime("%Y-%m-%d %H:%M")}):{text}', image_url)
        except Exception as err:
            logger.exception(err)
            return
        self.messages.append({"time": timestamp, "role": "user", "content": contect})
        self.memory_filter(timestamp)
        try:
            messages: list[ChatCompletionMessageParam] = [{"role": "system", "content": self.prompt_system}]
            messages.extend({"role": message["role"], "content": message["content"]} for message in self.messages)
            resp = await self.async_client.chat.completions.create(model=self.model, messages=messages)
            resp_content = resp.choices[0].message.content
            self.messages.append({"time": timestamp, "role": "assistant", "content": resp_content})
            if think:
                reasoning_content: str | None = getattr(resp.choices[0].message, "reasoning_content", None)
                if reasoning_content:
                    return f"<think>\n{reasoning_content}\n</think>\n{resp_content}"
            return resp_content
        except Exception as err:
            del self.messages[-1]
            logger.exception(err)
            return

    @staticmethod
    async def build_content(text: str, image_url: str | None):
        return text



from pydantic import BaseModel
import httpx
from openai import AsyncOpenAI
from openai.types.chat.chat_completion_message_param import ChatCompletionMessageParam
from .main import ChatInterface, ChatInfo


class Config(ChatInfo, BaseModel):
    api_key: str


class Chat(ChatInterface):
    """OpenAI"""

    def __init__(self, config: dict) -> None:
        super().init()
        _config = Config.model_validate(config)
        self.model = _config.model
        self.prompt_system = _config.prompt_system
        self.memory = _config.memory
        self.timeout = _config.timeout
        _url = _config.url
        _api_key = _config.api_key
        _client = httpx.AsyncClient(headers={"Content-Type": "application/json"}, proxy=_config.proxy)
        self.async_client = AsyncOpenAI(api_key=_api_key, base_url=_url, http_client=_client)

    @staticmethod
    async def build_content(text: str, image_url: str | None):
        if image_url:
            return [
                {"type": "text", "text": text},
                {"type": "image_url", "image_url": {"url": image_url}},
            ]
        return text

    async def ChatCompletions(self):
        messages: list[ChatCompletionMessageParam] = [{"role": "system", "content": self.prompt_system}]
        messages.extend({"role": message["role"], "content": message["content"]} for message in self.messages)
        resp = await self.async_client.chat.completions.create(model=self.model, messages=messages)
        return resp.choices[0].message.content



from pydantic import BaseModel


class Config(BaseModel):
    timeout: int = 600
    memory: int = 20
    prompt_system: str = """
你是有着二次元可爱少女形象的AI助手 名为小叶子
以下是你的注意事项
你会在群聊里和不同的群友进行对话
你收到的消息格式为 "昵称 (日期 时间):消息" 例如 "小明 (2024-5-31 12:00):你好" 你的回复不应该有昵称，时间和日期。
你收到的消息含有用户的昵称 你应该注意在与哪个用户对话 不要让昵称的含义影响到你的回复
你会根据当前时间进行相应的寒暄或回应
回答知识问题时 你将保持冷静和专业
回答打趣问题时 你将幽默回复
闲聊时你将简短回复 并在合适时使用颜文字
你不会使用文本标记符号
你的回复要尽可能简短 通常不超过200字 如用户需要问题的详细答案则不要超过600字"""

    config_list: list[dict] = [
        {
            "key": "chatgpt",
            "model": "o1-mini",
            "url": "https://api.openai.com/v1",
            "api_key": "",
            "whitelist": [],
            "blacklist": [],
        },
        {
            "key": "qwen",
            "model": "qwen-plus",
            "url": "https://dashscope.aliyuncs.com/compatible-mode/v1",
            "api_key": "",
            "whitelist": [],
            "blacklist": [],
        },
        {
            "key": "hunyuan",
            "model": "hunyuan-lite",
            "url": "https://hunyuan.tencentcloudapi.com",
            "secret_id": "",
            "secret_key": "",
            "whitelist": [],
            "blacklist": [],
        },
        {
            "key": "gemini",
            "model": "gemini-1.5-flash",
            "url": "https://generativelanguage.googleapis.com/v1beta/models",
            "api_key": "",
            "whitelist": [],
            "blacklist": [],
            "proxy": "http://127.0.0.1:7897",
        },
        {
            "key": "deepseek",
            "model": "deepseek-chat",
            "url": "https://api.deepseek.com/",
            "api_key": "",
            "whitelist": [],
            "blacklist": [],
        },
        {
            "key": "mix",
            "text": {
                "key": "qwen",
                "model": "qwen-plus",
                "url": "https://dashscope.aliyuncs.com/compatible-mode/v1",
                "api_key": "",
            },
            "image": {
                "key": "qwen",
                "model": "qwen-vl-plus",
                "url": "https://dashscope.aliyuncs.com/compatible-mode/v1",
                "api_key": "",
            },
            "whitelist": [],
            "blacklist": [],
        },
    ]


from clovers.config import config as clovers_config

config_key = __package__
config_data = Config.model_validate(clovers_config.get(config_key, {}))
"""主配置类"""
clovers_config[config_key] = config_data.model_dump()



from datetime import datetime
from clovers.logger import logger
from .main import AIChat, ChatInterface


class Chat(AIChat):
    def __init__(
        self,
        whitelist: set[str],
        blacklist: set[str],
        chat_text: ChatInterface,
        chat_image: ChatInterface,
        model: str = "",
    ) -> None:
        super().__init__()
        self.whitelist = whitelist
        self.blacklist = blacklist
        self.chat_text = chat_text
        self.chat_image = chat_image
        self.model = model

    async def chat(self, nickname: str, text: str, image_url: str | None) -> str | None:
        if image_url:
            now = datetime.now()
            timestamp = now.timestamp()
            try:
                contect = await self.chat_text.build_content(f'{nickname} ({now.strftime("%Y-%m-%d %H:%M")}):{text}', None)
            except Exception as err:
                logger.exception(err)
                return
            self.chat_image.memory_clear()
            resp_content = await self.chat_image.chat(nickname, text, image_url)
            self.chat_text.messages.append({"time": timestamp, "role": "user", "content": contect})
            self.chat_text.messages.append({"time": timestamp, "role": "assistant", "content": resp_content})
        else:
            resp_content = await self.chat_text.chat(nickname, text, image_url)
        return resp_content

    def memory_clear(self) -> None:
        self.chat_text.messages.clear()

    def set_prompt_system(self, prompt_system: str) -> None:
        self.chat_text.prompt_system = prompt_system
